//
//	ADUtilities.h
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <Foundation/NSString.h>
#import <Foundation/NSRange.h>
#import <stdio.h>

/*" Quick macro to set an object ivar safely without using autorelease. "*/
#define SETIVAR(ivar, newValue)	[newValue retain]; [ivar release]; ivar = newValue

@class NSCharacterSet, NSString, NSCalendarDate, NSArray;

/*" Logs to stdout "*/
void ADLog(NSString *format, ...);
void ADDebugLog(int debugLevel, NSString *format, ...);
void adfprintf(FILE *fp, const char *format, ...);

/*" Returns the search path used to look for template files "*/
extern NSArray *ADTemplateSearchPath();

/*"
 * Returns the NSCharacterSet for the allowable characters in a variable
 * name (letters, digits, and underscore).  ADNonVariableNameSet() returns
 * the inverted set of that.
"*/
extern NSCharacterSet *ADVariableNameSet();
extern NSCharacterSet *ADNonVariableNameSet();

/*"
 * Functions to determine if strings should be considered special types of
 * comments.  These are used by all the tokenizers, so I wanted to
 * centralize the functionality here.
"*/
extern BOOL ADIsCopyrightComment(NSString *commentString);
extern BOOL ADIsAutodocComment(NSString *commentString);

/*"
 * Finds the left parentheses matching the rightmost right parentheses.
 * Takes into account nested parentheses.
"*/
extern NSRange ADRangeOfMatchedLeftString(NSString *aString, NSString *leftChar, NSString *rightChar);
extern NSRange ADRangeOfMatchedLeftParen(NSString *aString);
extern NSRange ADRangeOfMatchedLeftSquareBracket(NSString *aString);

/*"
 * Finds the first variable name by searching backwards from lastIndex.
"*/
extern NSRange ADRangeOfVariableBeforeIndex(NSString *string, unsigned lastIndex);
extern NSRange ADRangeOfVariableInTypeString(NSString *string);

/*"
 * Almost like -stringBySquashingWhitespace, except it will leave a
 * trailing space if the original string had trailing whitespace.
"*/
extern NSString *ADSquashTypeString(NSString *aString);

/*"
 * Removes any extension from path, then returns it with extension added
 * on.
"*/
extern NSString *ADPathWithExtension(NSString *path, NSString *extension);

/*" Path normalization "*/
extern NSString *ADNormalizePath(NSString *path);

/*"
 * File existence functions.
"*/
extern BOOL ADFileExists(NSString *filename);
extern BOOL ADFileIsNewer(NSString *file1, NSString *file2);

/*"
 * Returns the array of words that should be bolded in the generated
 * documentation; used by ADFormatString().  The list is currently
 * hardcoded to be "self", "super", "id", and "nil".
"*/
extern NSArray *ADBoldWordsArray();

/*"
 * Formats the string, converting autodoc-style markup into the appropriate
 * output format.
"*/
extern NSString *ADFormatString(NSString *string, NSArray *italicWordsArray);

/*"
 * Special formatting function to do indenting.  Currently just a hack for
 * typedefs, and probably only works right for RTF.
"*/
extern NSString *ADFormatValueString(NSString *string);

