//
//	AutoDocMain.m
//	  Written by Carl Lindberg Copyright 1999 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADDefaults.h"
#import <Foundation/NSUtilities.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSAutoreleasePool.h>
#import <Foundation/NSFileManager.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSException.h>
#import <MiscFoundation.subproj/regexpr.h>
#import "ADOutputModule.h"
#import "ADUtilities.h"
#import "ADInputHandler.h"

void ADHandleInputFile(NSString *filename);
void ADSetupInputHandlers();
static NSMutableDictionary *inputHandlerDict;

#if LIB_FOUNDATION_LIBRARY
int main(int argc, char *argv[], char *env[])
#else
int main(int argc, char *argv[])
#endif
{
  NSAutoreleasePool *pool;
	NSEnumerator *fileEnum;
	NSString	 *filename;
	NSArray      *filesToProcess;
	NSDirectoryEnumerator *directoryEnum = nil;
	NSString	 *projectDirectory;
#if LIB_FOUNDATION_LIBRARY
        [NSProcessInfo initializeWithArguments:argv count:argc environment:env];
#endif

#if DEBUG
        NS_DURING {
#endif
        
        pool = [[NSAutoreleasePool alloc] init];
        filesToProcess = ADHandleArguments(argc, argv);
        projectDirectory = ADStringForDefault(ADProjectDirectory);

	re_set_syntax(RE_SYNTAX_AWK);
	ADSetupInputHandlers();

	fileEnum = [filesToProcess objectEnumerator];
	while ((filename = [fileEnum nextObject]))
	{
        if (!ADFileExists(filename))
            ADLog(@"Error: '%@' not found", filename);
        else
            ADHandleInputFile(filename);
	}

	if (projectDirectory != nil)
    {
		directoryEnum = [[NSFileManager defaultManager] enumeratorAtPath:projectDirectory];

        /* Recurse through entire project */
                while ((filename = [directoryEnum nextObject]))
        {
            NSString *fullFilename = [projectDirectory stringByAppendingPathComponent:filename];
            ADHandleInputFile(fullFilename);
        }
	}

	[pool release];
#if DEBUG
        }
        NS_HANDLER {
          fprintf(stderr, "catched: %s\n",
                  [[localException description] cString]);
          abort();
        }
        NS_ENDHANDLER;
#endif
	return 0;
}

void ADHandleInputFile(NSString *filename)
{
    id handler = [inputHandlerDict objectForKey:[filename pathExtension]];

	if (handler != nil)
    {
        [handler handleInputFile:filename];
    }
}

/*
 * FIXME it would be better to encapsulate this logic inside ADInputHandler
 * somewhere -- maybe even make it a class cluster.  It would also be
 * better to register any ADInputHandler subclasses found in the runtime,
 * but this is easier for now.
 */
#import "ADObjectiveCHandler.h"
#import "ADWebScriptHandler.h"
#import "ADCopyInputHandler.h"

void ADSetupInputHandlers()
{
	NSString *outputDirectory  = ADStringForDefault(ADOutputDirectory);
	BOOL  	 copyDocFiles = ADBoolForDefault(ADCopyDocumentation);

	inputHandlerDict = [[NSMutableDictionary alloc] init];

    [inputHandlerDict setObject:[ADObjectiveCHandler class] forKey:@"h"];
	[inputHandlerDict setObject:[ADWebScriptHandler class]  forKey:@"wos"];

	if (copyDocFiles && outputDirectory != nil)
    {
        NSArray      *docExtensions = [[ADOutputModule currentModule] documentationExtensions];
        NSEnumerator *extensionEnum = [docExtensions objectEnumerator];
		NSString     *currExtension;

		while ((currExtension = [extensionEnum nextObject]))
        {
            [inputHandlerDict setObject:[ADCopyInputHandler class] forKey:currExtension];
        }
    }

	ADDebugLog(3, @"Input handler dictionary: %@", inputHandlerDict);
}
