//
//    NSFileManager+Autodoc.m 
//      Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                       All rights reserved.
//        This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	
//  This file formatted with 4 spaces per tabstop.

#import "NSFileManager+Autodoc.h"
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSArray.h>

@implementation NSFileManager (Autodoc)

- (BOOL)directoryExistsAtPath:(NSString *)path
{
	BOOL isDir = NO;
	BOOL isFile = [self fileExistsAtPath:path isDirectory:&isDir];
	return (isFile && isDir);
}

- (BOOL)regularFileExistsAtPath:(NSString *)path
{
	BOOL isDir = NO;
	BOOL isFile = [self fileExistsAtPath:path isDirectory:&isDir];
	return (isFile && !isDir);
}

- (BOOL)deepCreateDirectoryAtPath:(NSString *)path attributes:(NSDictionary *)attributes
{
	NSString *parent = [path stringByDeletingLastPathComponent];

	if ([parent length] > 0 && ![self directoryExistsAtPath:parent])
	{
		// May want to ensure owner write permissions here...
		if (![self deepCreateDirectoryAtPath:parent attributes:attributes])
			return NO;
	}

	return [self createDirectoryAtPath:path attributes:attributes];
}

- (NSString *)findFile:(NSString *)filename inSearchPath:(NSArray *)paths
{
	int i, count = [paths count];

	for (i=0; i<count; i++)
	{
		NSString *currPath = [paths objectAtIndex:i];
		NSString *fullPath = [currPath stringByAppendingPathComponent:filename];

		if ([self regularFileExistsAtPath:fullPath])
			return fullPath;
	}

	return nil;
}

@end
