//
//  KeyValue+MiscMerge.m
//            Written by Don Yacktman and Carl Lindberg
//        Copyright 1998 by Don Yacktman and Carl Lindberg.
//                     All rights reserved.
//      This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "KeyValue+MiscMerge.h"
#import <Foundation/NSObjCRuntime.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSString.h>
#if GNU_RUNTIME
#  import <objc/objc-api.h>
#else
#  import <objc/objc-runtime.h>
#endif

@interface NSObject (WarningAvoidance)
+ (BOOL)accessInstanceVariablesDirectly;
@end

@implementation NSObject (MiscMergeHasKey)

- (BOOL)hasMiscMergeKeyPath:(NSString *)keyPath
{
	NSString *key = keyPath;
	NSRange dotRange = [keyPath rangeOfString:@"."];

	if (dotRange.length > 0)
	{
		key = [keyPath substringToIndex:dotRange.location];
	}
	
	return [self hasMiscMergeKey:key];
}

- (BOOL)hasMiscMergeKey:(NSString *)key
{
#if GNU_RUNTIME
  static Ivar_t class_getInstanceVariable(Class myClass, const char *ckey) {
    struct objc_ivar_list *ivars;
    Ivar_t ivar;
    int pos;

    if (myClass == Nil)  return NULL;
    if (ckey    == NULL) return NULL;
    
    for (pos = 0, ivar = NULL, ivars = myClass->ivars;
         pos < ivars->ivar_count; pos++) {
      Ivar_t internalIvar = &(ivars->ivar_list[pos]);

      if (strcmp(internalIvar->ivar_name, ckey) == 0) {
        ivar = internalIvar;
        break;
      }
    }
    return ivar;
  }
#endif
#if DEBUG
  if (key == nil)
    NSLog(@"WARNING: passed <nil>-key to %@", NSStringFromSelector(_cmd));
#endif
  if (key == nil)
    return NO;
	return [self respondsToSelector:NSSelectorFromString(key)] ||
				([[self class] accessInstanceVariablesDirectly] &&
				  class_getInstanceVariable([self class], [key cString]));
}

@end

@implementation NSDictionary (MiscMergeHasKey)

- (BOOL)hasMiscMergeKey:(NSString *)key
{
	return ([self objectForKey:key] != nil)? YES : NO;
}

@end

@implementation NSArray (MiscMergeHasKey)

- (BOOL)hasMiscMergeKey:(NSString *)key
{
	if ([key isEqualToString:@"count"]) return YES;
	if ([key hasPrefix:@"@"]) return YES;
	if ([self count] == 0) return YES; //Hmm
	return [[self objectAtIndex:0] hasMiscMergeKey:key];
}

@end
