//
//	ADClass.m
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADClass.h"
#import <Foundation/NSArray.h>
#import <Foundation/NSString.h>
#import "ADComment.h"
#import "ADUtilities.h"

@implementation ADClass
/*"
 * This class represents an Objective-C class parsed from the source files.
"*/


- init
{
	[super init];
	[self setDescriptionString:@"No class description."];
	return self;
}

- (void)dealloc
{
	[className release];
	[superclassName release];
	[inheritsInfo release];
	[ivarArray release];
	[super dealloc];
}

- (void)setClassName:(NSString *)aName			{	SETIVAR(className, aName);			}
- (void)setSuperclassName:(NSString *)aName		{	SETIVAR(superclassName, aName);		}
- (void)setIvarArray:(NSArray *)array			{	SETIVAR(ivarArray, array);			}
- (void)setInheritsInfo:(ADComment *)comment	{	SETIVAR(inheritsInfo, comment);		}

- (NSString *)className			{	return className;		}
- (NSString *)superclassName	{	return superclassName;	}
- (NSArray *)ivarArray			{	return ivarArray;		}
- (ADComment *)inheritsInfo		{	return inheritsInfo;	}

- (void)addIvarsFromArray:(NSArray *)array
{
	if ([ivarArray count] == 0)
	{
		[self setIvarArray:array];
	}
	else
	{
		[self setIvarArray:[ivarArray arrayByAddingObjectsFromArray:array]];
	}	
}

- (NSString *)title
{
	return [self className];
}

- (NSString *)interfaceTitle
{
	return @"Class";
}

- (NSString *)uniqueFileBasename
{
	return [self className];
}

- (NSString *)typeDirectoryName
{
    return @"Classes";
}

/*"
 * Returns the string that represents the continuation of the class'
 * heirarchy past the superclass, pulled from the inheritsInfo ivar.  If
 * not set, this returns an empty string, otherwise returns a string that
 * starts with " : " and ends with the contents of inheritsInfo.
"*/
- (NSString *)inheritsInfoString
{
	NSString *inheritString = [inheritsInfo commentString];
	
	if (inheritString == nil) return @"";

	if ([inheritString hasPrefix:@":"])
		return [@" " stringByAppendingString:inheritString];
	else
		return [NSString stringWithFormat:@": %@", inheritString];
}

- (NSString *)inheritsString
{
	return [NSString stringWithFormat:@"%@%@",
		superclassName ? superclassName : @"",
		inheritsInfo ?   [NSString stringWithFormat:@" %@", [self inheritsInfoString]] : @""];
}

- description
{
	NSString *superclassString;
	
	superclassString = [NSString stringWithFormat:@"%@%@",
		superclassName ? [NSString stringWithFormat:@" : %@", superclassName] : @"",
		inheritsInfo ?   [NSString stringWithFormat:@" %@", [self inheritsInfoString]] : @""];

	return [NSString stringWithFormat:@"@interface %@%@%@\n%@\n%@%@",
				className,
				superclassString,
				[self conformedProtocolString],
				[self ivarArray],
				[self entityDescription],
				[self declarationArray]];
}

@end
