//
//	ADHeaderEntity.m
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADHeaderEntity.h"

@implementation ADHeaderEntity

/*"
 * ADHeaderEntity is the abstract superclass of all the non-Objective-C
 * parts of a header file, e.g. macros, functions, global variables, etc.
 * These generally don't have much in common, except that they need to know
 * whether they should be grouped with a previous header entity under the
 * same description in the documentation, or whether they should be the
 * first entity in a new grouping.  This class provides the instance
 * variable to set this attribute.
 * 
 * The ADFile method -groupArrayFromEntityArray: and the ADFile methods
 * that use it are better interfaces to using this information, as those
 * perform the grouping logic.
"*/

/*"
 * Returns YES if the receiver should be grouped with the previous entity
 * of its type in the documentation.
"*/
- (BOOL)relatedToPreviousEntity
{
	return relatedToPreviousEntity;
}

/*"
 * Sets whether the receiver should be grouped with the previous entity in
 * the documentation.  Mainly for use by the parser.
"*/
- (void)setRelatedToPreviousEntity:(BOOL)yesno
{
	relatedToPreviousEntity = yesno;
}

@end

