//
//	ADMacro.m
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADMacro.h"
#import <Foundation/NSCharacterSet.h>
#import <Foundation/NSArray.h>
#import <MiscFoundation.subproj/NSString+MiscRegex.h>
#import <MiscFoundation.subproj/NSString+MiscAdditions.h>

@implementation ADMacro
/*"
 *  ADMacro represents a precompiler macro function.
"*/

/*" Creates a new, autoreleased ADMacro instance "*/
+ macroWithString:(NSString *)string
{
	return [[[self alloc] initWithString:string] autorelease];
}

/*"
 * Initializes a new ADMacro with string, which should contain the entire
 * macro definition.  The name and the parameters are parsed out, and the
 * actual definition is dropped.  The parameterString will not contain the
 * actual surrounding parentheses.
"*/
- initWithString:(NSString *)string
{
	NSRange		paramStartRange, paramEndRange;
	NSRange		nameStartRange, paramRange;
	NSString	*prefixString;
	
	[super init];

	paramStartRange = [string rangeOfString:@"("];
	paramEndRange = [string rangeOfString:@")"];
	
	paramRange.location = NSMaxRange(paramStartRange);
	paramRange.length = paramEndRange.location - paramRange.location;
	parameterString = [[string substringWithRange:paramRange] stringBySquashingWhitespace];

	prefixString = [string substringToIndex:paramStartRange.location];
	nameStartRange = [prefixString
						rangeOfCharacterFromSet:[NSCharacterSet whitespaceCharacterSet]
						options:NSBackwardsSearch];
	name = [prefixString substringFromIndex:NSMaxRange(nameStartRange)];


	[name retain];
	[parameterString retain];

	return self;
}

- (void)dealloc
{
	[name release];
	[parameterString release];
	[super dealloc];
}
	
- (NSString *)name
{
	return name;
}

- (NSString *)parameterString
{
	return parameterString;
}

/*" Override to return the list of parameters "*/
- (NSArray *)wordsToItalicize
{
	return [parameterString componentsSeparatedByRegex:@" *, *"];
}

- description
{
	return [NSString stringWithFormat:@"#define %@(%@)", name, parameterString];
}


@end
