//
//	ADTypedef.m
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADTypedef.h"
#import <Foundation/NSArray.h>
#import <MiscFoundation.subproj/NSString+MiscRegex.h>
#import <MiscFoundation.subproj/NSString+MiscAdditions.h>
#import "ADUtilities.h"


@implementation ADTypedef
/*"
 * ADTypedef represents a type definition declared in the header file.  It
 * stores the name, value (i.e. the thing being defined), and, if it's a
 * function typedef, the part that comes after the typedef name.
"*/

+ typedefWithString:(NSString *)string
{
	return [[[self alloc] initWithString:string] autorelease];
}

/*"
 * Intializes a new ADTypedef with string.  It parses the name, and the
 * value is everything that comes between the word "typedef" and the name.
 * If it's a function typedef, the stuff after the name is stored in
 * suffixString.
"*/
- initWithString:(NSString *)string
{
	NSRange		semicolonRange, nameRange, suffixRange;

	[super init];

	nameRange      = ADRangeOfVariableInTypeString(string);
	semicolonRange = [string rangeOfString:@";" options:NSBackwardsSearch];
	suffixRange.location = NSMaxRange(nameRange);
	suffixRange.length   = semicolonRange.location - suffixRange.location;
	
	name  = [string substringWithRange:nameRange];
	value = [string substringToIndex:nameRange.location];
	value = [value substringFromIndex:NSMaxRange([value rangeOfString:@"typedef"])];
	value = [value stringByTrimmingLeadWhitespace];
	suffixValue = [[string substringWithRange:suffixRange] stringBySquashingWhitespace];
	

	[name retain];
	[value retain];
	[suffixValue retain];

	return self;
}

- (void)dealloc
{
	[name release];
	[value release];
	[suffixValue release];
	[super dealloc];
}

- (NSString *)name
{
	return name;
}

- (NSString *)value
{
	return value;
}

- (NSString *)suffixValue
{
	return suffixValue;
}

- (NSString *)formattedValue
{
	return ADFormatValueString(value);
}

- description
{
	return [NSString stringWithFormat:@"typedef %@|%@|%@", value, name, suffixValue];
}

@end

