//
//    ADCopyInputHandler.m 
//      Written by Carl Lindberg Copyright 1999 by Carl Lindberg.
//                       All rights reserved.
//        This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	
//  This file formatted with 4 spaces per tabstop.

#import "ADCopyInputHandler.h"
#import <Foundation/NSFileManager.h>
#import <Foundation/NSPathUtilities.h>
#import "ADUtilities.h"
#import "ADDefaults.h"

@implementation ADCopyInputHandler

+ (void)handleInputFile:(NSString *)filename
{
	NSString *outFile = [self outputFilenameForInputFilename:filename];

	if ([[filename pathExtension] isEqual:@"html"] &&
		[[[filename stringByDeletingLastPathComponent] pathExtension] isEqual:@"wo"])
    {
		/* Don't copy over the html template files found inside WOComponents. */
        return;
    }

	/* Ensure that the output file has the same extension as the input file */
	outFile = ADPathWithExtension(outFile, [filename pathExtension]);

	if ([self shouldOutputToFile:outFile fromFile:filename])
    {
		[self createPathForOutputFile:outFile];
		if (!ADBoolForDefault(ADRunSilently))
			ADLog(@"Copying %@", [outFile lastPathComponent]);
		[[NSFileManager defaultManager] copyPath:filename toPath:outFile handler:nil];
	}
}

@end
