//
//    ADObjectiveCHandler.m 
//      Written by Carl Lindberg Copyright 1999 by Carl Lindberg.
//                       All rights reserved.
//        This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	
//  This file formatted with 4 spaces per tabstop.

#import "ADObjectiveCHandler.h"
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSAutoreleasePool.h>
#import "ADDefaults.h"
#import "ADUtilities.h"
#import "ADFile.h"

/* Defined in the parser (.y) files */
extern void ADParseHeaderFile(ADFile *fileContents);
extern void ADParseSourceFile(ADFile *fileContents);

@implementation ADObjectiveCHandler
/*"
 * ADObjectiveCHandler handles Objective-C source files.
"*/

/*"
 * The given filename should be the header file, and it will parse both the
 * .h and corresponding .m, .mdoc, or .M file in the same directory.
"*/
+ (void)handleInputFile:(NSString *)filename
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	BOOL	 parseSingles = ADBoolForDefault(ADParseSingleHeaders);
	BOOL	 outputAll    = ADBoolForDefault(ADDocumentAllClasses);
    BOOL     useTypeDirs  = ADBoolForDefault(ADUseDocTypeDirectories);
	NSString *headerFilename = ADPathWithExtension(filename, @"h");
	NSString *sourceFilename = ADPathWithExtension(filename, @"m");
	NSString *outputFilename = [self outputFilenameForInputFilename:filename];
	ADFile   *fileContents;

	if (!ADFileExists(headerFilename)) return;

	/* If the .m doesn't exist, check for a .mdoc in case this is a protocol, or then .M */
	if (!ADFileExists(sourceFilename)) sourceFilename = ADPathWithExtension(filename, @"mdoc");
	if (!ADFileExists(sourceFilename)) sourceFilename = ADPathWithExtension(filename, @"M");
	if (!ADFileExists(sourceFilename)) sourceFilename = nil;

	/* Bail if there's no implementation file and we're not parsing lone .h files */
	if (sourceFilename == nil && !parseSingles) return;

    fileContents = [[[ADFile alloc] init] autorelease];
    [fileContents setHeaderFilename:headerFilename];
    [fileContents setSourceFilename:sourceFilename];
    [fileContents setImportString:[headerFilename lastPathComponent]];

    /*
     * Avoid doing the parse if the file hasn't changed since we last
     * generated the docs.  Setting ADDocumentAllClasses or
     * ADUseDocTypeDirectories voids this optimization, since the output
     * filename can't be determined until after the source has been parsed.
     */
	if (!outputAll && !useTypeDirs &&
     ![self shouldOutputToFile:outputFilename fromADFile:fileContents])
	{
		return;
	}

	ADParseHeaderFile(fileContents);
	ADParseSourceFile(fileContents);
    [fileContents setHasParsed:YES];

	[self handleMainADFile:fileContents];
	
	[pool release];
}

@end
