//
//    ADWebScriptHandler.m 
//      Written by Carl Lindberg Copyright 1999 by Carl Lindberg.
//                       All rights reserved.
//        This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	
//  This file formatted with 4 spaces per tabstop.

#import "ADWebScriptHandler.h"
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSAutoreleasePool.h>
#import "ADDefaults.h"
#import "ADUtilities.h"
#import "ADFile.h"

/* Defined in the parser (.y) file */
extern void ADParseWebScriptFile(ADFile *fileContents);

@implementation ADWebScriptHandler
/*"
 * This class handles WebScript (*.wos) input files.
"*/


+ (void)handleInputFile:(NSString *)filename
{
	NSAutoreleasePool *pool  = [[NSAutoreleasePool alloc] init];
	NSString *outputFilename = [self outputFilenameForInputFilename:filename];
	BOOL	 outputAll       = ADBoolForDefault(ADDocumentAllClasses);
    BOOL     useTypeDirs     = ADBoolForDefault(ADUseDocTypeDirectories);
	ADFile	 *fileContents;

	if (!ADFileExists(filename)) return;

    /*
     * Avoid doing the parse if the file hasn't changed since we last
     * generated the docs.  Setting ADDocumentAllClasses or
     * ADUseDocTypeDirectories voids this optimization, since the output
     * filename can't be determined until after the source has been parsed.
     */
	if (!outputAll && !useTypeDirs &&
     ![self shouldOutputToFile:outputFilename fromFile:filename])
	{
		return;
	}

	fileContents = [[[ADFile alloc] init] autorelease];
	[fileContents setHeaderFilename:filename];
	[fileContents setSourceFilename:filename];
	[fileContents setImportString:[filename lastPathComponent]];

	ADParseWebScriptFile(fileContents);
    [fileContents setHasParsed:YES];
	
	[self handleMainADFile:fileContents];

	[pool release];
}

@end
