%{
//
//	HeaderGrammar.y
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

	#import <Foundation/NSDictionary.h>
	#import <Foundation/NSPathUtilities.h>
	#import <Foundation/NSUtilities.h>
	#import <Foundation/NSArray.h>
	#import <MiscFoundation.subproj/NSString+MiscRegex.h>
	#import <stdio.h>
	#import <stdlib.h>
	#import <string.h>

	#import "ADClass.h"
	#import "ADComment.h"
	#import "ADProtocol.h"
	#import "ADCompoundIvar.h"
	#import "ADCategory.h"
	#import "ADMethod.h"
	#import "ADMacro.h"
	#import "ADConstant.h"
	#import "ADGlobalVariable.h"
	#import "ADInlineFunction.h"
	#import "ADTypedef.h"
	#import "ADFile.h"
	#import "ADDefaults.h"
	#import "ADUtilities.h"

	/* Is there a better way to handle this? */
	#undef yylex
	#define yylex HeaderTokenizerlex
	
	/* Use our logging function, so we can control where output goes */
	#define fprintf adfprintf

	/* Define some values, since we can't use the '@' character inside bison rules */
	#define EMPTYSTRING @""
	#define UNDERSCORE  @"_"
	#define UNIONSTRING @"union"
	#define STRUCTSTRING @"struct"
	
	typedef struct _MethodArg {
		NSString *argName;
		NSString *argType;
		NSString *varName;
	} MethodArg;
	
	
	int yyerror(char *s);
	int yylex();

	static ADHeaderEntity *lastEntity = nil;
	static ADComment *currentComment = nil;

	#define YYDEBUG 1
	#define YYERROR_VERBOSE 1
	
//	YYPARSE_PARAM doesn't work because of a snafu in bison.simple
//	#define YYPARSE_PARAM fileContents
//	#define ADFILE (ADFile *)fileContents
	static ADFile *globalFileContents;
	#define ADFILE (ADFile *)globalFileContents
%}

%expect 1

%token INTERFACE
%token PROTOCOL
%token END
%token BEGINUNION
%token BEGINSTRUCT
%token <stringVal> ENDSTRUCT
%token <stringVal> IDENT
%token <stringVal> AUTODOCCOMMENT
%token <stringVal> COPYRIGHTCOMMENT
%token <stringVal> STATICTYPE
%token <stringVal> VARDECL
%token <stringVal> FUNCTION
%token <stringVal> INLINEFUNCTION
%token <stringVal> GLOBALVAR
%token <stringVal> CONSTANT
%token <stringVal> MACRO
%token <stringVal> TYPEDEF
%token <stringVal> ENUM
%token <stringVal> ELLIPSIS
%token <charVal> METHODSTART

%type <method>method
%type <method>arguments
%type <methodArg>argument
%type <ivar>vardecl
%type <mutableArray>manymethods
%type <mutableArray>vardecls
%type <mutableArray>manyvardecls
%type <mutableArray>conforms
%type <mutableArray>manyconforms
%type <comment>maybecomment
%type <stringVal>statictype
%type <stringVal>maybeellipsis
%type <stringVal>maybeident
%type <object>filepiece

%union
{
	NSString *stringVal;
	int		intVal;
	char	charVal;
	MethodArg  methodArg;
	NSArray  *array;
	NSMutableArray *mutableArray;
	id	 object;
	ADMethod *method;
	ADIvar *ivar;
	ADComment *comment;
}

%%

file:	/* Empty */
		| file filepiece
		{
			if ([$2 isKindOfClass:[ADHeaderEntity class]])
			{
				if (currentComment)
				{
					[$2 setEntityDescription:currentComment];
					currentComment = nil;
				}
				else if ([$2 isMemberOfClass:[lastEntity class]])
				{
					[$2 setRelatedToPreviousEntity:YES];
				}
				lastEntity = $2;
			}
		}
		;

filepiece:  INTERFACE IDENT ':' IDENT conforms maybecomment vardecls manymethods END
			{
				$$ = [[[ADClass alloc] init] autorelease];
				[$$ setClassName:$2];
				[$$ setSuperclassName:$4];
				[$$ setConformedProtocols:$5];
				[$$ setInheritsInfo:$6];
				[$$ setIvarArray:$7];
				[$$ addDeclarationsFromArray:$8];
				[ADFILE addClass:$$];
			}
			/* A root class */
			| INTERFACE IDENT conforms vardecls manymethods END
			{
				$$ = [[[ADClass alloc] init] autorelease];
				[$$ setClassName:$2];
				[$$ setConformedProtocols:$3];
				[$$ setIvarArray:$4];
				[$$ addDeclarationsFromArray:$5];
				[ADFILE addClass:$$];
			}
			| INTERFACE IDENT '(' IDENT ')' conforms manymethods END
			{
				$$ = [ADCategory category];
				[$$ setClassName:$2];
				[$$ setCategoryName:$4];
				[$$ setConformedProtocols:$6];
				[$$ addDeclarationsFromArray:$7];
				[ADFILE addCategory:$$];
			}
			| PROTOCOL IDENT conforms manymethods END
			{
				$$ = [ADProtocol protocol];
				[$$ setProtocolName:$2];
				[$$ setConformedProtocols:$3];
				[$$ addDeclarationsFromArray:$4];
				[ADFILE addProtocol:$$];
			}
			| AUTODOCCOMMENT
			{
				$$ = [ADComment commentWithString:$1];
				currentComment = $$;
			}
			| COPYRIGHTCOMMENT
			{
				$$ = nil;
				[ADFILE setCopyrightFromString:$1];
			}
			| FUNCTION
			{
				$$ = [ADFunction functionWithString:$1];
				[ADFILE addFunction:$$];
			}
			| INLINEFUNCTION
            {
				$$ = [ADInlineFunction functionWithString:$1];
                [ADFILE addInlineFunction:$$];
			}
			| GLOBALVAR
			{
				$$ = [ADGlobalVariable variableWithString:$1];
				[ADFILE addVariable:$$];
			}
			| CONSTANT
			{
				$$ = [ADConstant constantWithString:$1];

                /*
                 * Often, headers are protected against multiple inclusion
                 * by something like the following:

					#ifndef _FOO_H_
					#define _FOO_H_
					...
					#endif _FOO_H_

                 * To avoid documenting these, we will not document a macro
                 * that starts with an underscore ("_") unless it has been
                 * explicitly documented.
                 */
				if (currentComment != nil || ![[$$ name] hasPrefix:UNDERSCORE])
				{
					[ADFILE addConstant:$$];
				}
			}
			| MACRO
			{
				$$ = [ADMacro macroWithString:$1];
				[ADFILE addMacro:$$];
			}
			| TYPEDEF
			{
				$$ = [ADTypedef typedefWithString:$1];
				[ADFILE addTypedef:$$];
			}
			| ENUM
			{
				/* Not handling these yet */
				$$ = nil;
			}
			;

conforms:	/* Empty */ { $$ = [NSMutableArray array]; }
			|	'<' manyconforms '>'  { $$ = $2; }
			;

manyconforms:	IDENT { $$ = [NSMutableArray arrayWithObject:$1]; }
			|   IDENT ',' manyconforms  { $$ = $3; [$$ addObject:$1]; }
			;

vardecls:	/* Empty */       { $$ = [NSMutableArray array]; }
			| '{' manyvardecls '}'  { $$ = $2; };

manyvardecls:	/* Empty */       { $$ = [NSMutableArray array]; }
			| manyvardecls vardecl  { $$ = $1; if ($2) [$$ addObject:$2]; };

vardecl:	VARDECL maybecomment
			{
				// Do "INTERNAL" check here
				$$ = [ADIvar ivarWithString:$1];
				if ($2) [$$ setEntityDescription:$2];
			}
			| BEGINUNION manyvardecls ENDSTRUCT
			{
				$$ = [ADCompoundIvar ivarWithString:$3];
				[(ADCompoundIvar *)$$ setSubIvars:$2];
				[(ADCompoundIvar *)$$ setType:UNIONSTRING];
			}
			| BEGINSTRUCT manyvardecls ENDSTRUCT
			{
				$$ = [ADCompoundIvar ivarWithString:$3];
				[(ADCompoundIvar *)$$ setSubIvars:$2];
				[(ADCompoundIvar *)$$ setType:STRUCTSTRING];
			}
			| error { $$ = nil; }
			;

maybecomment: /* Empty */  {$$ = nil; }
			| AUTODOCCOMMENT { $$ = [ADComment commentWithString:$1]; }
			;
			
manymethods:	/* Empty */       { $$ = [NSMutableArray array]; }
			| manymethods method  { $$ = $1; if ($2) [$$ addObject:$2]; };

method:		METHODSTART statictype IDENT arguments maybeellipsis ';'
			{
				$$ = $4;
				[$$ setClassMethod:($1 == '+')];
				[$$ setReturnType:$2];
				[$$ setFirstArgumentName:$3];
				if ([$5 length] > 0)
					[$$ setIsVarargMethod:YES];
			}
			| AUTODOCCOMMENT
			{
				$$ = (ADMethod *)[ADComment commentWithString:$1];
			}
			| error { $$ = nil; }

;

			/* Vararg method decls will end with ", ...", so try to deal with that. */
maybeellipsis:	/* Empty */	{$$ = EMPTYSTRING;}
			| ',' ELLIPSIS	{$$ = $2; }
			;

statictype:	/* Empty */ {$$ = EMPTYSTRING; }
			| STATICTYPE {$$ = $1; }
			;

arguments:	/* Empty */
			{
				$$ = [ADMethod method];
			}
			| arguments argument
			{
				$$ = $1;
				[$$ addArgument:$2.argName type:$2.argType varName:$2.varName];
			}
;

argument:	maybeident ':' statictype IDENT
			{
				$$.argName = $1;
				$$.argType = $3;
				$$.varName = $4;
			}
			| error
		 	{
				$$.argName = EMPTYSTRING;
				$$.argType = EMPTYSTRING;
				$$.varName = EMPTYSTRING;
			}
			;

maybeident: /* Empty */   { $$ = EMPTYSTRING; }
		  | IDENT         { $$ = $1; }
		  ;

%%

extern void ADSetHeaderFile(FILE *file);
extern int  ADHeaderLineNumber();

void ADParseHeaderFile(ADFile *fileContents)
{
	FILE *headerFile;
	int debugLevel = ADIntForDefault(ADDebugLevel);
	
#if YYDEBUG != 0
	yydebug = (debugLevel >= 5);
#endif

	if ([fileContents headerFilename])
	{
		headerFile = fopen([[fileContents headerFilename] cString], "r");
		if (headerFile)
		{
			ADDebugLog(1, @"Parsing %@...\n", [fileContents headerFilename]);
			ADSetHeaderFile(headerFile);
			globalFileContents = fileContents;
			lastEntity = nil;
			currentComment = nil;
			yyparse();
			fclose(headerFile);
			headerFile = NULL;
		}
	}
}


int yyerror(char *s)
{
	ADLog(@"Parse error: %@: %d: %s\n", [ADFILE headerFilename], ADHeaderLineNumber(), s);
	return 1;
}

