
%{
	#import <Foundation/NSDictionary.h>
	#import <Foundation/NSPathUtilities.h>
	#import <Foundation/NSUtilities.h>
	#import <Foundation/NSArray.h>
	#import <MiscFoundation.subproj/NSString+MiscRegex.h>
	#import <stdio.h>
	#import <stdlib.h>
	#import <string.h>

	#import "ADClass.h"
	#import "ADComment.h"
	#import "ADCategory.h"
	#import "ADMethod.h"
	#import "ADFile.h"
	#import "ADIvar.h"
	#import "ADDefaults.h"
	#import "ADUtilities.h"

	/* Is there a better way to handle this? */
	#undef yylex
	#define yylex WebScriptTokenizerlex
	
	/* Use our logging function, so we can control where output goes */
	#define fprintf adfprintf

	/* Define some values, since we can't use the '@' character inside bison rules */
	#define EMPTYSTRING @""
	#define NSSTRING(cstring) [NSString stringWithCString:cstring]
	
	typedef struct _MethodArg {
		NSString *argName;
		NSString *argType;
		NSString *varName;
	} MethodArg;
	
	
	int yyerror(char *s);
	int yylex();

	#define YYDEBUG 1
	#define YYERROR_VERBOSE 1
	
//	YYPARSE_PARAM doesn't work because of a snafu in bison.simple
//	#define YYPARSE_PARAM fileContents
//	#define ADFILE (ADFile *)fileContents
	static ADFile *globalFileContents;
	#define ADFILE (ADFile *)globalFileContents

    /*
     * WOScript files don't have to formally declare the class they're
     * defining -- the class is assumed to be the same name as the input
     * file less the ".wos", and it's (usually) assumed to inherit from
     * WOComponent.  Any bare variable declarations are instance variables
     * of this class, and any bare method implmentations are methods of it.
     * The "globalClass" variable holds this implied class, and we add the
     * ivars and methods as they're parsed.  This class is the
     * -mainInterface of the ADFile, unless no methods were implemented for
     * it, in which case we assume everything is inside
     * @interface/@implementation sections and therefore ignore this
     * assumed class.
     */	   
	static ADClass *globalClass;
%}

%expect 6

%token INTERFACE
%token IMPLEMENTATION
%token END
%token <stringVal> IDENT
%token <stringVal> AUTODOCCOMMENT
%token <stringVal> COPYRIGHTCOMMENT
%token <stringVal> STATICTYPE
%token <stringVal> ELLIPSES
%token <stringVal> SQUIGGLEBLOCK
%token <charVal> METHODSTART

%type <method>methodimpl
%type <method>methoddecl
%type <method>arguments
%type <methodArg>argument
%type <ivar>ivar
%type <mutableArray>manymethodimpls
%type <mutableArray>manymethoddecls
%type <mutableArray>ivardecllist
%type <mutableArray>ivarlist
%type <mutableArray>vardecl
%type <mutableArray>vardecls
%type <mutableArray>manyvardecls
%type <mutableArray>conforms
%type <mutableArray>manyconforms
%type <comment>maybecomment
%type <stringVal>statictype
%type <stringVal>maybeellipses
%type <stringVal>maybeident
%type <object>filepiece
%type <object>filepieces
%type <object>implstart

%union
{
	NSString *stringVal;
	int		intVal;
	char	charVal;
	MethodArg  methodArg;
	NSArray  *array;
	NSMutableArray *mutableArray;
	id	 object;
	ADMethod *method;
	ADIvar *ivar;
	ADComment *comment;
}

%%

file:	/* Empty */
		| maybecopyrights maybecomment filepieces
        {
			/* A comment at the beginning of the file is the class description */
			if ($2) [globalClass setEntityDescription:$2];
		}
        ;

maybecopyrights: /* Empty */
			   | COPYRIGHTCOMMENT maybecopyrights
			   {
                   [ADFILE setCopyrightFromString:$1];
			   }
			   ;

filepieces: filepiece
		  | filepieces filepiece
		  ;

filepiece:  INTERFACE IDENT ':' IDENT conforms maybecomment vardecls manymethoddecls END
			{
				$$ = [[[ADClass alloc] init] autorelease];
				[$$ setClassName:$2];
				[$$ setSuperclassName:$4];
				[$$ setConformedProtocols:$5];
				[$$ setInheritsInfo:$6];
				[$$ setIvarArray:$7];
				[$$ addDeclarationsFromArray:$8];
				[ADFILE addClass:$$];
			}
			/* A root class */
			| INTERFACE IDENT conforms vardecls manymethoddecls END
			{
				$$ = [[[ADClass alloc] init] autorelease];
				[$$ setClassName:$2];
				[$$ setConformedProtocols:$3];
				[$$ setIvarArray:$4];
				[$$ addDeclarationsFromArray:$5];
				[ADFILE addClass:$$];
			}
			| INTERFACE IDENT '(' IDENT ')' conforms manymethoddecls END
			{
				$$ = [ADCategory category];
				[$$ setClassName:$2];
				[$$ setCategoryName:$4];
				[$$ setConformedProtocols:$6];
				[$$ addDeclarationsFromArray:$7];
				[ADFILE addCategory:$$];
			}
			| COPYRIGHTCOMMENT
			{
				$$ = nil;
				[ADFILE setCopyrightFromString:$1];
			}
			| methodimpl
			{
				if ($1) [globalClass addDeclaration:$1];
			}
			| vardecl
			{
				if ($1) [globalClass addIvarsFromArray:$1];
			}
			/* An @implementation section */
			| implstart maybecomment manymethodimpls END
			{
				if ($2) [$1 setEntityDescription:$2];
				
                /*
                 * If there were methods declared in the @interface
                 * section, then we should just document those.  If there
                 * were none, as is common, we'll document all methods
                 * found while parsing the @implementation.
                 */
				if ([[$1 declarationArray] count] > 0)
					[$1 setMethodDescriptionsFromArray:$3];
				else
					[$1 addDeclarationsFromArray:$3];
			}
			;

implstart:  IMPLEMENTATION IDENT ':' IDENT vardecls
			{
				$$ = [ADFILE classNamed:$2];
			}
			| IMPLEMENTATION IDENT vardecls
			{
				$$ = [ADFILE classNamed:$2];
			}
			| IMPLEMENTATION IDENT '(' IDENT ')'
			{
				$$ = [ADFILE categoryNamed:$4 forClass:$2];
				
                /*
                 * If a category had no @interface section, which is
                 * common, then add the category.  In fact, I'm not sure
                 * WebScript even allows @interface sections for
                 * categories, in which case this is absolutely needed.
                 */
				if ($$ == nil)
				{
					$$ = [ADCategory category];
					[$$ setClassName:$2];
					[$$ setCategoryName:$4];
					[ADFILE addCategory:$$];
				}
			}
			;

conforms:	/* Empty */ { $$ = [NSMutableArray array]; }
			|	'<' manyconforms '>'  { $$ = $2; }
			;

manyconforms:	IDENT                   { $$ = [NSMutableArray arrayWithObject:$1]; }
			|   IDENT ',' manyconforms  { $$ = $3; [$$ addObject:$1]; }
			;

vardecls: /* Empty */           { $$ = [NSMutableArray array]; }
		| '{' manyvardecls '}'  { $$ = $2; };

manyvardecls:	/* Empty */       { $$ = [NSMutableArray array]; }
			| manyvardecls vardecl  { $$ = $1; if ($2) [$$ addObjectsFromArray:$2]; };

vardecl:	ivardecllist maybecomment
			{
				$$ = $1;
				if ($2) [[$$ objectAtIndex:0] setEntityDescription:$2];
			}
			;

ivardecllist: IDENT ivarlist ';'
			{
				int i, count = [$2 count];
				$$ = $2;
				
				for (i=0; i<count; i++)
				{
					ADIvar *ivar = [$$ objectAtIndex:i];
					NSString *type = $1;

					/* If we assigned a partial type string below, include it. */
					if ([ivar type] != nil)
						type = [type stringByAppendingString:[ivar type]];
					
					[ivar setType:type];
				}
			}
			;
				

ivarlist: ivar               { $$ = [NSMutableArray arrayWithObject:$1]; }
        | ivarlist ',' ivar  { $$ = $1; [$$ addObject:$3]; }
		| error { $$ = nil; }
		;

ivar: IDENT     { $$ = [ADIvar ivarWithName:$1]; [$$ setType:NSSTRING(" ")];  }
    | '*' IDENT { $$ = [ADIvar ivarWithName:$2]; [$$ setType:NSSTRING(" *")]; }
	;

maybecomment: /* Empty */  {$$ = nil; }
			| AUTODOCCOMMENT { $$ = [ADComment commentWithString:$1]; }
			;
			
manymethoddecls:  /* Empty */                 { $$ = [NSMutableArray array]; }
				| manymethoddecls methoddecl  { $$ = $1; if ($2) [$$ addObject:$2]; };

methoddecl: METHODSTART statictype IDENT arguments maybeellipses ';'
			{
				$$ = $4;
				[$$ setClassMethod:($1 == '+')];
				[$$ setReturnType:$2];
				[$$ setFirstArgumentName:$3];
				if ([$5 length] > 0)
					[$$ setIsVarargMethod:YES];
			}
			| AUTODOCCOMMENT
			{
				$$ = (ADMethod *)[ADComment commentWithString:$1];
			}
			| error { $$ = nil; }
			;

manymethodimpls:  /* Empty */       		 { $$ = [NSMutableArray array]; }
				| manymethodimpls methodimpl { $$ = $1; if ($2) [$$ addObject:$2]; }
				;

methodimpl:	maybecomment METHODSTART statictype IDENT arguments maybeellipses maybesemicolon
			maybecomment SQUIGGLEBLOCK
			{
				$$ = $5;
				[$$ setClassMethod:($2 == '+')];
				[$$ setReturnType:$3];
				[$$ setFirstArgumentName:$4];
				if ([$6 length] > 0)
					[$$ setIsVarargMethod:YES];

				/* A comment inside the method block will win if both are present */
				if ($1) [$$ setEntityDescription:$1];
				if ($8) [$$ setEntityDescription:$8];
			}
			| error {$$ = nil; }

maybesemicolon: /* Empty */
			  | ';'
			  ;

			/* Vararg method decls will end with ", ...", so try to deal with that. */
maybeellipses:	/* Empty */	{$$ = EMPTYSTRING;}
			 | ',' ELLIPSES	{$$ = $2; }
			 ;

statictype:	/* Empty */ {$$ = EMPTYSTRING; }
			| STATICTYPE {$$ = $1; }
			;

arguments:	/* Empty */
			{
				$$ = [ADMethod method];
			}
			| arguments argument
			{
				$$ = $1;
				[$$ addArgument:$2.argName type:$2.argType varName:$2.varName];
			}
;

argument:	maybeident ':' statictype IDENT
			{
				$$.argName = $1;
				$$.argType = $3;
				$$.varName = $4;
			}
			| error
		 	{
				$$.argName = EMPTYSTRING;
				$$.argType = EMPTYSTRING;
				$$.varName = EMPTYSTRING;
			}
			;

maybeident: /* Empty */   { $$ = EMPTYSTRING; }
		  | IDENT         { $$ = $1; }
		  ;

%%

extern void ADSetWebScriptFile(FILE *file);
extern int  ADWebScriptLineNumber();

void ADParseWebScriptFile(ADFile *fileContents)
{
	FILE *wosFile;
	int debugLevel = ADIntForDefault(ADDebugLevel);
	
#if YYDEBUG != 0
	yydebug = (debugLevel >= 5);
#endif

	if ([fileContents headerFilename])
	{
		wosFile = fopen([[fileContents headerFilename] cString], "r");
		if (wosFile)
		{
			ADDebugLog(1, @"Parsing %@...", [fileContents headerFilename]);
			ADSetWebScriptFile(wosFile);
			globalFileContents = fileContents;
			globalClass = [[ADClass alloc] init];
			yyparse();
			fclose(wosFile);
			wosFile = NULL;

            /*
             * If there were any methods implemented on the assumed class,
             * then it becomes the mainInterface.  Otherwise, if no methods
             * were declared, then we ignore it and let the first
             * @interface section stay as the mainInterface.
             */
			if (![fileContents hasMainInterface] || [[globalClass declarationArray] count] > 0)
			{
				NSString *filename  = [[fileContents headerFilename] lastPathComponent];
				NSString *className = [filename stringByDeletingPathExtension];
	
				[globalClass setClassName:className];
				if ([className isEqual:@"Application"])
					[globalClass setSuperclassName:@"WOApplication"];
				else if ([className isEqual:@"Session"])
					[globalClass setSuperclassName:@"WOSession"];
				else
					[globalClass setSuperclassName:@"WOComponent"];
	
				[fileContents addClass:globalClass];
				[fileContents setMainInterface:globalClass];
			}

			[globalClass release];
			globalClass = nil;
		}
	}
}


int yyerror(char *s)
{
    // We get errors when parsing WOS files using the "modern" syntax, and
    // possibly at other times when there's valid syntax, so we're not
    // logging errors unless debugging is turned on.
	ADDebugLog(1, @"Parse error: %@: %d: %s\n",
		[ADFILE headerFilename], ADWebScriptLineNumber(), s);

	return 1;
}

