
%{

#import <Foundation/NSString.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSData.h>
#import <MiscFoundation.subproj/NSString+MiscRegex.h>
#import <MiscFoundation.subproj/NSString+MiscAdditions.h>

#import "ADClass.h"
#import "ADComment.h"
#import "ADProtocol.h"
#import "ADIvar.h"
#import "ADCategory.h"
#import "ADMethod.h"
#import "ADDefaults.h"
#import "ADUtilities.h"

#define YYSTRING [NSString stringWithCString:yytext length:yyleng]

	typedef struct _MethodArg {
		NSString *argName;
		NSString *argType;
		NSString *varName;
	} MethodArg;

#import "WebScriptGrammar.m.h"

#define yylval WebScriptGrammarlval

/* Use our logging function, so we can control where output goes */
#define fprintf adfprintf

static int numSquiggles = 0;
static int numParens = 0;
static int currlineno;

static void UpdateLineno(char *text, int textLength);
static void SetupWebScriptTokenizer();
#define YY_USER_ACTION UpdateLineno(yytext, yyleng);
#define YY_USER_INIT   SetupWebScriptTokenizer();

%}

%option debug
%option never-interactive
%option noyywrap
%option stack
%option noyy_top_state
%option noreject
%option nounput

ident				[a-zA-Z_][a-zA-Z0-9_]*
ccomment			("/*"([^*]|"*"+[^*/])*"*"+"/")
objccomment			("//"[^\n]*\n?)
objccommentblock	{objccomment}([ \t]*{objccomment})*
anycomment			{ccomment}|{objccomment}
whitespace			[ \t\r\n]*
conststring         ["](\\.|[^\\"])*["]
constchar			['](\\.|[^\\'])[']
statictype			(\([^/:;@+{}-]*\))
methodstart			^{whitespace}[+-]{whitespace}{statictype}?
methodargument		":"{whitespace}{statictype}?{whitespace}{ident}


%s IN_INTERFACE
%s IN_IMP
%s METHOD
%s VARIABLES
%x SQUIGGLES
%x ASSIGNVAL

%%

{ccomment}			|
{objccommentblock}	{
						NSString *commentString = YYSTRING;
						
						if (YYSTATE == INITIAL && ADIsCopyrightComment(commentString))
						{
							yylval.stringVal = commentString;
							return COPYRIGHTCOMMENT;
						}

						if (ADIsAutodocComment(commentString))
						{
							yylval.stringVal = commentString;
							return AUTODOCCOMMENT;
						}
					}


@interface				{	BEGIN(IN_INTERFACE); return INTERFACE; }
@implementation			{	BEGIN(IN_IMP); return IMPLEMENTATION; }

<IN_INTERFACE,IN_IMP>[:()<>,]	{	return yytext[0]; }

<IN_INTERFACE,IN_IMP>@end		{	BEGIN(INITIAL); return END;	}

<IN_INTERFACE>"{"				{	BEGIN(VARIABLES); return yytext[0]; }

<VARIABLES>"}"			{	BEGIN(IN_INTERFACE); return yytext[0]; }

						/* Maybe we shouldn't document @private ivars at all */
<VARIABLES>(@public|@protected|@private)	;

<IN_INTERFACE,IN_IMP,INITIAL>[+-]		{
							yy_push_state(METHOD);
							yylval.charVal = yytext[0];
							return METHODSTART;
						}

<METHOD>[:,]			{	return yytext[0]; }

<METHOD>{statictype}	{	yylval.stringVal = YYSTRING; return STATICTYPE;	}
<METHOD>"..."			{	yylval.stringVal = YYSTRING; return ELLIPSES; }

<METHOD>";"				{	yy_pop_state(); return yytext[0]; }

<METHOD,IN_IMP,INITIAL>"{"	{
   								if (YYSTATE == METHOD)
									yy_pop_state();
                            	yy_push_state(SQUIGGLES);
								numSquiggles++;
							}

<SQUIGGLES>{anycomment}		;
<SQUIGGLES>{conststring}	;
<SQUIGGLES>{constchar}		;
<SQUIGGLES>"{"			{	numSquiggles++; }
<SQUIGGLES>"}"			{	numSquiggles--; 
							if (numSquiggles == 0)
							{
								yy_pop_state();
								return SQUIGGLEBLOCK;
							}
						}
<SQUIGGLES>[^{}/"']+	;
<SQUIGGLES>[/]			;  // A slash that's not the start of a comment

        /*
         * In WebScript, variables can be initialized to values, so ignore
         * those if we find them.
         */
"="							{	yy_push_state(ASSIGNVAL); numSquiggles = numParens = 0;	}
<ASSIGNVAL>{anycomment}		;
<ASSIGNVAL>{conststring}	;
<ASSIGNVAL>"{"				{	numSquiggles++;	}
<ASSIGNVAL>"}"				{	numSquiggles--;	}
<ASSIGNVAL>"("				{	numParens++;	}
<ASSIGNVAL>")"				{	numParens--;	}
<ASSIGNVAL>[^{}()/";]+	;
<ASSIGNVAL>[/]			;  // A slash that's not the start of a comment
<ASSIGNVAL>";"				{
								if (numSquiggles == 0 && numParens == 0)
								{
									yy_pop_state();
									return yytext[0];
								}
							}

[*,;]	 			    {	return yytext[0]; }
{ident}					{	yylval.stringVal = YYSTRING; return IDENT;	}

{whitespace}				;
.|\n						;


%%

static void UpdateLineno(char *text, int textLength)
{
	int i;

	for (i=0; i<textLength; i++)
		if (text[i] == '\n') currlineno++;
}

static void SetupWebScriptTokenizer()
{
#ifdef FLEX_DEBUG
	yy_flex_debug = (ADIntForDefault(ADDebugLevel) >= 4);
#endif
}

void ADSetWebScriptFile(FILE *file)
{
	numSquiggles = 0;
	numParens = 0;
	currlineno = 1;
	yyin = file;
}

int ADWebScriptLineNumber()
{
	return currlineno;
}
