#import <Foundation/NSRunLoop.h>
#import <Foundation/NSAutoreleasePool.h>
#import <Foundation/NSException.h>
#import <AppKit/NSApplication.h>
#import "FormatComment.h"

/*
 * I prefer Command-/ as the command key for this, but ProjectBuilder's
 * Stop Build command takes it, so FormatComment declares Command-. as its
 * key equivalent.  [On NT, Ctrl is treated like the Command key...]
 * 
 * This is overridable, usally.  On OPENSTEP, you can use the NSCommandKeys
 * dwrite or just use the Menu preferences module to force "Format Comment"
 * to use "/".
 * 
 * On MacOS X, you can use the NSUserKeyEquivalents dwrite something like
 * the below to re-wire the key equivalents.  However, this trick doesn't
 * seem to quite work under NT, at least for commands in the Services menu.
 * Still I'm leaving this comment here in case someone does figure it out
 * someday, and so I can remember the syntax for the NSUserKeyEquivalents
 * default :-)
 * 
 * To use, just do
 * 
 * defaults write ProjectBuilder NSUserKeyEquivalents '"Format Comment" =
 * "@/";'
 * 
 * on the command line.  For key equivalents, @ is command, ^ is control, $
 * is shift, and ~ is alt.  A & in the menu title string means the _ is put
 * under the following letter for NT menus.
 * 
 * NSUserKeyEquivalents = {"&Stop Build" = "@("; "Format Comment" = "@/"; };
 */


int main(int argc, const char *argv[])
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    FormatComment *serviceProvider = [[FormatComment alloc] init];

    NSRegisterServicesProvider(serviceProvider, @"FormatComment");

    NS_DURING
        [[NSRunLoop currentRunLoop] configureAsServer];
        [[NSRunLoop currentRunLoop] run];
    NS_HANDLER
        NSLog(@"%@", localException);
    NS_ENDHANDLER

    [serviceProvider release];
    [pool release];

    exit(0);       // insure the process exit status is 0
    return 0;      // ...and make main fit the ANSI spec.
}
