//
//	ADFunction.m
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADFunction.h"
#import <Foundation/NSArray.h>
#import <Foundation/NSUtilities.h>
#import <MiscFoundation.subproj/NSString+MiscAdditions.h>
#import "ADUtilities.h"

NSRange leftParen(NSString *aString);

@implementation ADFunction
/*" Represents an "externed" C function. "*/

+ functionWithString:(NSString *)string
{
	return [[[self alloc] initWithString:string] autorelease];
}

/*"
 * Takes string, which should contain the entire "extern" definition, and
 * parses out the return type, name, and parameters.  The return type does
 * not include the "extern" word, and the parameterString includes
 * everything between the parentheses but not the parentheses themselves.
"*/
- initWithString:(NSString *)string
{
	NSRange		rightParenRange, leftParenRange;
	NSRange		parenRange, nameRange, typeRange;

	[super init];

	rightParenRange = [string rangeOfString:@")" options:NSBackwardsSearch];
	leftParenRange = ADRangeOfMatchedLeftParen(string);

	parenRange.location = NSMaxRange(leftParenRange);
	parenRange.length = rightParenRange.location - parenRange.location;
	parameterString = [[string substringWithRange:parenRange]
                                   stringBySquashingWhitespace];
	
	nameRange = ADRangeOfVariableBeforeIndex(string, leftParenRange.location);
	name = [string substringWithRange:nameRange];
	
	typeRange.location = NSMaxRange([string rangeOfString:[string wordNum:0]]);
	typeRange.length = nameRange.location - typeRange.location;
	type = [string substringWithRange:typeRange];
	type = ADSquashTypeString(type);
	
	[name retain];
	[type retain];
	[parameterString retain];

	return self;
}

- (void)dealloc
{
	[name release];
	[type release];
	[parameterString release];
	[super dealloc];
}
	
- (NSString *)name
{
	return name;
}

- (NSString *)type
{
	return type;
}

- (NSString *)parameterString
{
	return parameterString;
}

- (NSArray *)wordsToItalicize
{
	NSArray *paramArray = [parameterString componentsSeparatedByString:@","];
	NSMutableArray *italArray = [NSMutableArray arrayWithCapacity:[paramArray count]];
	NSEnumerator *paramEnum = [paramArray objectEnumerator];
	NSString *paramString;
	NSRange variableRange;
	
	while ((paramString = [paramEnum nextObject]))
	{
		if ([paramString length] > 0)
		{
			variableRange = ADRangeOfVariableBeforeIndex(paramString, [paramString length]);
			if (variableRange.length > 0)
				[italArray addObject:[paramString substringWithRange:variableRange]];
		}
	}
	
	return italArray;
}

- description
{
	return [NSString stringWithFormat:@"extern %@%@(%@)", type, name, parameterString];
}

@end

